from typing import TypedDict, Union, Optional, List, Dict, Any


class SSHKeyAuth(TypedDict):
    r"""
- Contains SSH connection details for key authentication

- It is used as an _input_ to many of the actions in this plugin

- The example below shows the dictionary structure in YAML

!!! warning "Supplying Credentials"

    You should ___always___ use a PPA Vault integration to provide credentials to a plugin action.

!!! example

    ```yaml linenums="1"
    ssh_server:
      address: 1.2.3.4
      port: 22
      username: [username]
      private_key: [private_key_string]
      passphrase: [optional_passphrase]
      base_prompt: "cisco\-\d{3}#"
    ```

???+ danger "Required Keys"

    __address__: The target IP or DNS address.

    __port__: The target SSH port.

    __username__: Username for authentication.

    __private_key__: RSA private key for authentication.

    __passphrase__: The passphrase for the supplied _private_key_ ___or___ _null_ if not required.

    __base_prompt__: A regular expression representing the SSH command prompt.
    """

    address: str
    port: Union[str, int]
    username: str
    private_key: Optional[str]
    passphrase: Optional[str]
    base_prompt: Optional[str]


class SSHPasswordAuth(TypedDict):
    r"""
- Contains SSH connection details for password authentication

- It is used as an _input_ to many of the actions in this plugin

- The example below shows the dictionary structure in YAML

!!! warning "Supplying Credentials"

    You should ___always___ use a PPA Vault integration to provide credentials to a plugin action.

!!! example

    ```yaml linenums="1"
    ssh_server:
      address: 1.2.3.4
      port: 22
      username: [username]
      password: [password]
      base_prompt: "cisco\-\d{3}#"
    ```


???+ danger "Required Keys"

    __address__: The target IP or DNS address.

    __port__: The target SSH port.

    __username__: Username for authentication.

    __password__: Password for authentication.

    __base_prompt__: A regular expression representing the SSH command prompt.

    """
    address: str
    port: Union[str, int]
    username: str
    password: Optional[str]
    base_prompt: Optional[str]


SSHServer = Union[SSHPasswordAuth, SSHKeyAuth]


class SCPServer(TypedDict):
    r"""
- Contains connection details for an SCP server

- The example below shows the dictionary structure in YAML

!!! warning "Supplying Credentials"

    You should ___always___ use a PPA Vault integration to provide credentials to a plugin action.

!!! example

    ```yaml linenums="1"
    scp_server:
      address: 1.2.3.4
      port: 22
      username: [username]
      password: [password]
    ```


???+ danger "Required Keys"

    __address__: The target IP or DNS address.

    __port__: The target SCP port.

    __username__: Username for authentication.

    __password__: Password for authentication.

    """
    address: str
    port: Union[str, int]
    username: str
    password: Optional[str]


class CommandQuestion(TypedDict):
    r"""
- Contains prompts and responses for questions asked by a command

- You can supply as many of these questions & answers as you need to

- Each question prompt ___must___ be unique per-command

- Question prompts that aren't found are ignored

__Question Handling__

The following process is started if question prompt is found.

1. The answer to the prompted question is sent
2. If the response is in `base_prompt` or _success_, processing is __complete__
3. Or if it's in another question prompt, go back to __step 1__

If a response isn't found in step 2 or 3 it will be treated as a failure.

!!! example
    {% raw %}
    ```YAML linenums="1" hl_lines="4 5 6"
    - ssh.client.run_commands:
        commands:
          - command: sudo ls --color=never -lsah
            questions:
              prompt: "\[sudo\] password for .*:"
              answer: "{{ ssh_server.password }}"
      load:
        ssh_server: ssh_server
      save: directory_listing
    ```
    {% endraw %}

???+ danger "Required Keys"

    __prompt__: A regular expression matching the question prompt

    __answer__: The command to run if the question is asked
    """

    prompt: str
    answer: str


class Command(TypedDict):
    """
- Contains command information for the [run_commands](../ssh.client/#run_command) action

!!! example

    ```YAML linenums="1" hl_lines="2"
    - ssh.client.run_command:
        command: ls --color=never -lsah
      load:
        ssh_server: ssh_server
      save: directory_listing
    ```

???+ danger "Required Keys"

    __command__: The command to run

???+ info "Optional Keys"

    __success__: A regular expression matching the prompt when the command succeeds (defaults to the _base_prompt_)

    __questions__: A list of [CommandQuestion](#commandquestion) dictionaries (defaults to _null_)
    """

    command: str
    success: Optional[str]
    questions: Optional[List[Dict[str, Any]]]
